package com.madindo.trivia;

import android.content.SharedPreferences;
import android.graphics.Color;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.CardView;
import android.util.Log;
import android.view.View;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.madindo.trivia.data.AnswerListAsyncResponse;
import com.madindo.trivia.model.Question;
import com.madindo.trivia.model.QuestionBank;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity implements View.OnClickListener {

    private TextView questionTextview;
    private TextView questionCounterTextview;
    private Button trueButton;
    private Button falseButton;
    private ImageButton nextButton;
    private ImageButton prevButton;
    private int currentQuestionIndex = 0;
    private List<Question> questionList;
    private static final String MESSAGE_ID = "highscore_prefs";
    private int currentScore = 0;
    private int highScore = 0;
    private TextView pointTextview;
    private TextView highTextView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        SharedPreferences getSharePrefs = getSharedPreferences(MESSAGE_ID, MODE_PRIVATE);
        int highscore = getSharePrefs.getInt("highscore", 0);

        nextButton = findViewById(R.id.next_button);
        prevButton = findViewById(R.id.prev_button);
        trueButton = findViewById(R.id. true_button);
        falseButton = findViewById(R.id.false_button);
        questionCounterTextview = findViewById(R.id.counter_text);
        questionTextview = findViewById(R.id.question_textview);
        pointTextview = findViewById(R.id.point_textview);
        highTextView = findViewById(R.id.highscore_textview);

        highTextView.setText("Highscore : " + highscore);


        nextButton.setOnClickListener(this);
        prevButton.setOnClickListener(this);
        trueButton.setOnClickListener(this);
        falseButton.setOnClickListener(this);

        questionList = new QuestionBank().getQuestions(new AnswerListAsyncResponse() {
            @Override
            public void processFinished(ArrayList<Question> questionArrayList) {
                questionTextview.setText(questionArrayList.get(currentQuestionIndex).getAnswer());
                //Log.d("Inside", "processFinished: " + questionArrayList);
            }
        });
        //Log.d("Main", "onCreate: " + questionList);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.prev_button:
                if (currentQuestionIndex > 0) {
                    currentQuestionIndex--;
                    updateCounter();
                    updateQuestion();
                }
                break;
            case R.id.next_button:
                if (currentQuestionIndex < 100) {
                    currentQuestionIndex++;
                    updateCounter();
                    updateQuestion();
                }

                break;
            case R.id.true_button:
                checkAnswer(true);
                updateQuestion();
                break;
            case R.id.false_button:
                checkAnswer(false);
                updateQuestion();
                break;
        }
    }

    private void updateQuestion() {
        String question = questionList.get(currentQuestionIndex).getAnswer();
        questionTextview.setText(question);
    }

    private void updateCounter() {
        questionCounterTextview.setText(currentQuestionIndex + " out of 100");
    }

    private void updateCurrentPoint() {
        pointTextview.setText("Point : " + currentScore);
    }

    private void checkAnswer(Boolean userChooseCorret){
        boolean answerIsTrue = questionList.get(currentQuestionIndex).isAnswerTrue();
        int toastMessageId = 0;
        if (answerIsTrue == userChooseCorret) {
            fadeView();
            toastMessageId = R.string.correct_answer;
            if (currentScore >= 0 && currentScore < 100) {
                currentScore++;
            }

        } else {
            shakeAnimation();
            toastMessageId = R.string.wrong_answer;
            if (currentScore > 0 && currentScore <= 100) {
                currentScore--;
            }
        }
        updateCurrentPoint();

        currentQuestionIndex++;
        updateCounter();
        updateQuestion();

        Toast.makeText(this, toastMessageId, Toast.LENGTH_SHORT).show();
        if (currentScore > highScore) {
            SharedPreferences sharedPreferences = getSharedPreferences(MESSAGE_ID, MODE_PRIVATE);
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putInt("highscore", currentScore);
            editor.apply();
        }

    }

    private void shakeAnimation(){
        Animation shake = AnimationUtils.loadAnimation(MainActivity.this, R.anim.shake_animation);
        final CardView cardView = findViewById(R.id.cardView);
        cardView.setAnimation(shake);
        shake.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {
                cardView.setCardBackgroundColor(Color.RED);
                questionTextview.setTextColor(Color.WHITE);
            }

            @Override
            public void onAnimationEnd(Animation animation) {
                cardView.setCardBackgroundColor(Color.WHITE);
                questionTextview.setTextColor(Color.BLACK);
            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }
        });
    }

    private void fadeView() {
        final CardView cardView = findViewById(R.id.cardView);
        AlphaAnimation alphaAnimation = new AlphaAnimation(1.0f, 0.0f);
        alphaAnimation.setDuration(350);
        alphaAnimation.setRepeatCount(1);
        alphaAnimation.setRepeatMode(Animation.REVERSE);

        cardView.setAnimation(alphaAnimation);

        alphaAnimation.setAnimationListener(new Animation.AnimationListener() {
            @Override
            public void onAnimationStart(Animation animation) {
                cardView.setBackgroundColor(Color.GREEN);
            }

            @Override
            public void onAnimationEnd(Animation animation) {
                cardView.setBackgroundColor(Color.WHITE);
            }

            @Override
            public void onAnimationRepeat(Animation animation) {

            }
        });
    }
}
